// -*- C++ -*-
//
//  This file is part of CBCBSIM, the Combined Broadcast Content-Based
//  (CBCB) routing simulation.
//
//  Authors: See the file AUTHORS for full details. 
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: CBCBDataCollector.h,v 1.1 2004/03/19 17:04:48 carzanig Exp $
//
#ifndef CBCBSIM_CBCB_DATA_COLLECTOR_H
#define CBCBSIM_CBCB_DATA_COLLECTOR_H

#include <cbnsim/DataCollector.h>
#include "cbcbsim/CBCBNode.h"

namespace cbcbsim
{
  class CBCBDataCollector : public cbnsim::DataCollector
    {
      unsigned int  m_inl_count;
      unsigned int  m_tot_count;
    
    public:
      CBCBDataCollector() : m_inl_count( 0 ), m_tot_count( 0 ) {}
  
      void poll_node( cbnsim::Node& node )
	{
	  CBCBNode& n = static_cast<CBCBNode&>( node );
	  m_inl_count += n.srur_packets();
	  m_tot_count += n.srur_packets();
	  n.reset_srur_packets();
	}

      void output_inline( std::ostream& out, ssim::Time time )
	{
	  out << "INL_SRUR_COUNT\t" << time << "\t" << m_inl_count << std::endl;
	  m_inl_count = 0;
	}

      void output_final( std::ostream& out )
      {
	out << "FINAL_SRUR_COUNT\t" << m_tot_count << std::endl;
      }
    };
};

#endif
