// -*- C++ -*-
//
//  This file is part of CBCBSIM, the Combined Broadcast Content-Based
//  (CBCB) routing simulation.
//
//  Authors: See the file AUTHORS for full details. 
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: ProtocolOptions.h,v 1.2 2008-04-01 17:02:51 carzanig Exp $
//
#ifndef CBCBSIM_PROTOCOLOPTIONS_H
#define CBCBSIM_PROTOCOLOPTIONS_H

#include <siena/ssim.h>

namespace cbcbsim
{
  /** @brief Encapsulates the various protocol options that are available.
   */
  class ProtocolOptions
  {
  public:
    enum PredicateIssuePolicy { PIP_IMMEDIATE, PIP_TIMED };
    enum PredicateRefreshPolicy { PRP_NEVER, PRP_TIMED };
    enum SRSendPolicy { SRSP_NEVER, SRSP_TIMED, SRSP_TIMED_COUNT };

    ProtocolOptions()
    {
      pip = PIP_IMMEDIATE; pip_sec = 0; pip_time = 0;
      prp = PRP_TIMED; prp_sec = 600; prp_time = 0;
      srsp = SRSP_TIMED; srsp_count = 0; srsp_sec = 480; srsp_time = 0;
    }

    PredicateIssuePolicy	pip;
    ssim::Time			pip_sec;
    ssim::Time			pip_time;
    PredicateRefreshPolicy	prp;
    ssim::Time			prp_sec;
    ssim::Time			prp_time;
    SRSendPolicy		srsp;
    unsigned int		srsp_count;
    ssim::Time			srsp_sec;
    ssim::Time			srsp_time;
  };
};
#endif
