// -*- C++ -*-
//
//  This file is part of CBCBSIM, the Combined Broadcast Content-Based
//  (CBCB) routing simulation.
//
//  Authors: See the file AUTHORS for full details. 
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: CBCBSimulation.cc,v 1.1 2004/03/19 17:04:47 carzanig Exp $
//
#include <cmath> // for rint()
#include <unistd.h> // for getcwd()
#include <cbnsim/SendMessageParser.h>
#include <cbnsim/SetPredicateParser.h>
#include <cbnsim/logging.h>
#include <stdexcept>
#include "cbcbsim/CBCBDataCollector.h"
#include "cbcbsim/CBCBNodeFactory.h"
#include "cbcbsim/CBCBSimulation.h"

using namespace cbcbsim;
using namespace cbnsim;
using namespace std;

CBCBSimulation :: CBCBSimulation()
{
  add_event_parser( new SendMessageParser() );
  add_event_parser( new SetPredicateParser() );

  set_factory( new CBCBNodeFactory( m_bl, m_options ) );

  set_data_collector( new CBCBDataCollector() );
}

void CBCBSimulation :: setup_topology_hook() throw ()
{
  // finalize the broadcast layer
  m_bl.computePaths();

  // compute the time delays for the protocol options
  m_options.pip_time = (ssim::Time)rint( m_options.pip_sec / time_unit() );
  m_options.prp_time = (ssim::Time)rint( m_options.prp_sec / time_unit() );
  m_options.srsp_time = (ssim::Time)rint( m_options.srsp_sec / time_unit() );
}

void CBCBSimulation :: pre_run_simulation_hook() throw ()
{
  char cwd[PATH_MAX];
  getcwd( cwd, PATH_MAX );
  CBNDEBUG( "*************************************************" );
  CBNDEBUG( "cbcbsim invoked with the following settings:" );
  CBNDEBUG( "cwd = " << cwd );
  CBNDEBUG( "debug = " << debug_messages() );
  CBNDEBUG( "dci = " << dci() << " (sec)" );
  CBNDEBUG( "info = " << info_messages() );
  CBNDEBUG( "outfile = " << output_fname() );

  if( m_options.pip == ProtocolOptions::PIP_IMMEDIATE )
    {
      CBNDEBUG( "pip = immediate" );
    }
  else if( m_options.pip == ProtocolOptions::PIP_TIMED )
    {
      CBNDEBUG( "pip = timed " << m_options.pip_sec << " (sec), "
		<< m_options.pip_time << " (ticks)" );
    }
  else
    {
      throw logic_error( "unrecognized PIP" );
    }
  
  if( m_options.prp == ProtocolOptions::PRP_NEVER )
    {
      CBNDEBUG( "prp = never" );
    }
  else if( m_options.prp == ProtocolOptions::PRP_TIMED )
    {
      CBNDEBUG( "prp = timed " << m_options.prp_sec << " (sec), "
		<< m_options.prp_time << " (ticks)" );
    }
  else
    {
      throw logic_error( "unrecognized PRP" );
    }
  
  if( m_options.srsp == ProtocolOptions::SRSP_NEVER )
    {
      CBNDEBUG( "srsp = never" );
    }
  else if( m_options.srsp == ProtocolOptions::SRSP_TIMED )
    {
      CBNDEBUG( "srsp = timed " << m_options.srsp_sec << " (sec), "
		<< m_options.srsp_time << " (ticks)" );
    }
  else if( m_options.srsp == ProtocolOptions::SRSP_TIMED_COUNT )
    {
      CBNDEBUG( "srsp = timed-count " << m_options.srsp_sec << " (sec), "
		<< m_options.srsp_time << " (ticks)" );
    }
  else
    {
      throw logic_error( "unrecognized SRSP" );
    }
  
  CBNDEBUG( "wtu = " << time_unit() );
  CBNDEBUG( "workload = " << input_fname() );
  CBNDEBUG( "*************************************************" );
}
